<?php

namespace SheHeader\Modules\Transparent;

use Elementor;
// use Elementor\Core\Breakpoints\Manager as Breakpoints_manager;
use Elementor\Plugin;
use Elementor\Controls_Manager;
use Elementor\Controls_Stack;
use SheHeader\Base\Module_Base;

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class Module extends Module_Base
{

	public function __construct()
	{
		parent::__construct();

		$this->add_actions();
	}

	public function get_name()
	{
		return 'transparent';
	}

	public function register_controls(Controls_Stack $element)
	{

		// $active_breakpoints = [];
		// $active_devices = [];
		// $device_options = [];

		// 	// Active breakpoints
		// $this->active_breakpoints = Plugin::$instance->breakpoints->get_active_breakpoints();

		// // Elementor version >= 3.4.0
		// if(version_compare(ELEMENTOR_VERSION, '3.4.0', '>=')) {
		// 	// Add 'desktop' device
		// 	$args = ['add_desktop' => true, 'reverse' => true];

		// 	// Devices list
		// 	$this->active_devices = Plugin::$instance->breakpoints->get_active_devices_list($args);
		// } else {		
		//     // Devices need to be ordered from largest to smallest.
		//     $this->active_devices = array_reverse(array_keys($this->active_breakpoints));

		//     // Add desktop in the correct position.
		//     if(in_array('widescreen', $this->active_devices, true)) {
		//     	$this->active_devices = array_merge(array_slice($this->active_devices, 0, 1), ['desktop'], array_slice($this->active_devices, 1));
		//     } else {
		//     	$this->active_devices = array_merge(['desktop'], $this->active_devices);
		//     }
		// }

		$element->start_controls_section(
			'section_sticky_header_effect',
			[
				'label' => __('Sticky Header Effects', 'she-header'),
				'tab' => Controls_Manager::TAB_ADVANCED,
			]
		);

		$element->add_control(
			'transparent',
			[
				'label' => __('Enable', 'she-header'),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => __('On', 'she-header'),
				'label_off' => __('Off', 'she-header'),
				'return_value' => 'yes',
				'default' => 'no',
				'frontend_available' => true,
				'prefix_class'  => 'she-header-',
			]
		);

		$element->add_control(
			'sticky_header_notice',
			[
				'raw' => __('IMPORTANT: This plugin is NOT meant to make the header sticky, only to make changes when scrolling.<br>
				Use <b>Motion Effects > Sticky > Top</b> to make the header sticky initially
				<br>
				Most effects show only on the front end', 'she-header'),
				'type' => Controls_Manager::RAW_HTML,
				'content_classes' => 'elementor-panel-alert elementor-panel-alert-info',
				'condition' => [
					'transparent' => 'yes',
				],
			]
		);

		// Control options
		// foreach($this->active_devices as $device) {
		// 	$label = 'desktop' === $device ? esc_html__('Desktop', 'eac-components') : $this->active_breakpoints[$device]->get_label();
		// 	$this->device_options[$device] = $label;
		// }

		/** Apply breakpoints */
		// $element->add_control('transparent_on',
		// 	[
		// 	'label'			=> __('Enable On', 'she-header'),
		// 	'type'			=> Controls_Manager::SELECT2,
		// 	'multiple'		=> true,
		// 	'label_block'	=> true,
		// 	'default'		=> $this->active_devices, // vs [ 'desktop', 'tablet', 'mobile' ... ]
		// 	'options'		=> $this->device_options,
		// 	'condition' => [
		// 		'transparent!' => '',
		// 	],
		// 	'render_type' => 'none',
		// 	'description' => __('This will completely enable/disable ALL settings for each option except for sticky settings under the motion effects tab above', 'she-header'),
		// 	'frontend_available' => true,
		// ]
		// );

		$element->add_control(
			'transparent_on',
			[
				'label' => __('Enable On', 'she-header'),
				'type' => Controls_Manager::SELECT2,
				'multiple' => true,
				'label_block' => 'true',
				'default' => ['desktop', 'tablet', 'mobile'],
				'options' => [
					'desktop' => __('Desktop', 'she-header'),
					'tablet' => __('Tablet', 'she-header'),
					'mobile' => __('Mobile', 'she-header'),
				],
				'condition' => [
					'transparent!' => '',
				],
				'render_type' => 'none',
				'description' => __('This will completely enable/disable ALL settings for each option except for sticky settings under the motion effects tab above', 'she-header'),
				'frontend_available' => true,
			]
		);

		$element->add_responsive_control(
			'scroll_distance',
			[
				'label' => __('Scroll Distance (px)', 'she-header'),
				'type' => Controls_Manager::SLIDER,
				'default' => [
					'size' => 60,
				],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 500,
					],
				],
				'size_units' => ['px'],
				'description' => __('Choose the scroll distance to enable Sticky Header Effects', 'she-header'),
				'frontend_available' => true,
				'condition' => [
					'transparent!' => '',
				],
			]
		);

		$element->add_control(
			'settings_notice',
			[
				'raw' => __('Remember: The settings below will not be applied until the page is scrolled the distance set above', 'she-header'),
				'type' => Controls_Manager::RAW_HTML,
				'content_classes' => 'elementor-panel-alert',
				'condition' => [
					'transparent!' => '',
				],
			]
		);

		// ---------------------------------- TRANSPARENT SETTINGS
		$element->add_control(
			'transparent_header',
			[
				'label' => __('Transparent Header', 'she-header'),
				'type' => Controls_Manager::SWITCHER,
				'separator' => 'before',
				'label_on' => __('On', 'she-header'),
				'label_off' => __('Off', 'bew-header'),
				'return_value' => 'yes',
				'default' => '',
				'frontend_available' => true,
				'prefix_class'  => 'she-header-transparent-',
				'condition' => [
					'transparent!' => '',
				],
				'selectors' => [
					'{{WRAPPER}}.she-header-yes:not(.elementor-sticky--active)' => 'position: absolute;',
				],
				'description' => __('Sets the header position to "absolute" so negative margins are not needed', 'she-header'),
			]
		);

		$element->add_control(
			'transparent_spacing_note',
			[
				'raw' => __('This will make the header overlap the main page so extra spacing at the top of sections may be needed', 'she-header'),
				'type' => Controls_Manager::RAW_HTML,
				'content_classes' => 'elementor-panel-alert',
				'condition' => [

					'transparent!' => '',
					'transparent_header' => 'yes',
				],
			]
		);

		$element->add_control(
			'transparent_background',
			[
				'label' => __('Fully Transparent', 'she-header'),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => __('On', 'she-header'),
				'label_off' => __('Off', 'she-header'),
				'return_value' => 'yes',
				'default' => 'yes',
				'frontend_available' => true,
				'condition' => [
					'transparent!' => '',
					'transparent_header' => 'yes',
				],
				'selectors' => [
					'{{WRAPPER}}.she-header-yes:not(.elementor-motion-effects-element-type-background), {{WRAPPER}}.she-header-yes > .elementor-motion-effects-container > .elementor-motion-effects-layer' => 'background-color: rgba(0,0,0,0) !important;',
				],
			]
		);

		$element->add_control(
			'transparent_background_notice',
			[
				'raw' => __('Now you can use the background settings under the style tab', 'she-header'),
				'type' => Controls_Manager::RAW_HTML,
				'content_classes' => 'elementor-panel-alert',
				'condition' => [
					'transparent!' => '',
					'transparent_header' => 'yes',
					'transparent_background' => '',
				],
			]
		);

		$element->add_control(
			'transparent_background_description',
			[
				'raw' => __('Show or hide the background color', 'she-header'),
				'type' => Controls_Manager::RAW_HTML,
				'content_classes' => 'elementor-descriptor',
				'condition' => [
					'transparent!' => '',
					'transparent_header' => 'yes',
				],
			]
		);

		$element->add_control(
			'transparent_background_tip',
			[
				'raw' => __('Tip: This can be useful for creating initial semi-transparent headers or regular, non-transparent, sticky headers for Elementor free users', 'she-header'),
				'type' => Controls_Manager::RAW_HTML,
				'content_classes' => 'elementor-descriptor',
				'condition' => [
					'transparent!' => '',
					'transparent_header' => 'yes',
				],
			]
		);

		// ---------------------------------- BACKGROUND COLOR
		$element->add_control(
			'custom_background',
			[
				'label' => __('Background Color', 'she-header'),
				'type' => Controls_Manager::SWITCHER,
				'separator' => 'before',
				'label_on' => __('On', 'she-header'),
				'label_off' => __('Off', 'she-header'),
				'return_value' => 'yes',
				'default' => '',
				'frontend_available' => true,
				'prefix_class'  => 'she-header-custom-background-',
				'condition' => [
					'transparent!' => '',
				],
				'description' => __('Choose what color to change the background to after scrolling', 'she-header'),
			]
		);

		$element->add_control(
			'background',
			[
				'label' => __('Color', 'she-header'),
				'type' => Controls_Manager::COLOR,
				'default' => '',
				'title' => __('Background Color', 'she-header'),
				'selectors' => [
					'{{WRAPPER}}.she-header:not(.elementor-motion-effects-element-type-background), {{WRAPPER}}.she-header > .elementor-motion-effects-container > .elementor-motion-effects-layer' => 'opacity: 1 !important; background-color: {{VALUE}} !important;',
				],
				'condition' => [
					'custom_background' => 'yes',
					'transparent!' => '',
				],
			]
		);

		// ---------------------------------- BOTTOM BORDER
		$element->add_control(
			'bottom_border',
			[
				'label' => __('Bottom Border', 'she-header'),
				'type' => Controls_Manager::SWITCHER,
				'separator' => 'before',
				'label_on' => __('On', 'she-header'),
				'label_off' => __('Off', 'she-header'),
				'return_value' => 'yes',
				'default' => '',
				'frontend_available' => true,
				'condition' => [
					'transparent!' => '',
				],
				'description' => __('Choose bottom border size and color', 'she-header'),
				'selectors' => [
					'{{WRAPPER}}' => 'border-bottom-style: solid !important;',
					'{{WRAPPER}}.she-header' => 'border-style: solid !important;',
				],
			]
		);

		$element->add_control(
			'custom_bottom_border_color',
			[
				'label' => __('Color', 'she-header'),
				'type' => Controls_Manager::COLOR,
				'condition' => [
					'bottom_border' => 'yes',
					'transparent!' => '',
				],
				'render_type' => 'none',
				'frontend_available' => true,
				'selectors' => [
					'.she-header-yes' => 'border-bottom-color: rgba(0,0,0,0);',
					'{{WRAPPER}}.she-header' => 'border-bottom-color: {{VALUE}} !important;',
				],
			]
		);

		$element->add_control(
			'custom_bottom_border_width',
			[
				'label' => __('Bottom Border Thickness (Px)', 'she-header'),
				'type' => Controls_Manager::SLIDER,
				'default' => [
					'size' => 0,
				],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'size_units' => ['px'],
				'condition' => [
					'bottom_border' => 'yes',
					'transparent!' => '',
				],
				'frontend_available' => true,
				'description'        => __('Tip: To remove an existing border, set the color to transparent and the width to 0px', 'she-header'),
				'selectors' => [
					'.she-header-yes' => 'border-width: 0;',
					'{{WRAPPER}}.she-header' => 'border-width: 0 0 {{SIZE}}{{UNIT}} 0 !important;',
				],
			]
		);

		// ---------------------------------- SHRINK HEADER
		$element->add_control(
			'shrink_header',
			[
				'label' => __('Shrink Header', 'she-header'),
				'type' => Controls_Manager::SWITCHER,
				'separator' => 'before',
				'label_on' => __('On', 'she-header'),
				'label_off' => __('Off', 'she-header'),
				'return_value' => 'yes',
				'default' => '',
				'frontend_available' => true,
				'prefix_class'  => 'she-header-shrink-header-',
				'description' => __('Choose header height after scrolling', 'she-header'),
				'condition' => [
					'transparent!' => '',
				],
			]
		);

		$element->add_responsive_control(
			'custom_height_header',
			[
				'label' => __('Header Height (Px)', 'she-header'),
				'type' => Controls_Manager::SLIDER,
				'size_units' => ['px'],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 1000,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 70,
				],
				'description' => __('Remember: The header cannot shrink smaller than the elements inside of it', 'she-header'),
				'condition' => [
					'shrink_header' => 'yes',
					'transparent!' => '',
				],
				'frontend_available' => true,
				'selectors' => [
					'.she-header, .she-heaer.elementor-sticky__spacer' => 'min-height:{{SIZE}}{{UNIT}}, height:{{SIZE}}{{UNIT}} !important;',
				],
			]
		);

		// ---------------------------------- SHRINK LOGO
		$element->add_control(
			'shrink_header_logo',
			[
				'label' => __('Shrink Logo', 'she-header'),
				'type' => Controls_Manager::SWITCHER,
				'separator' => 'before',
				'label_on' => __('On', 'she-header'),
				'label_off' => __('Off', 'she-header'),
				'return_value' => 'yes',
				'default' => '',
				'frontend_available' => true,
				'description' => __('Resize logo after scrolling', 'she-header'),
				'condition' => [
					'transparent!' => '',
				],
			]
		);

		$element->add_control(
			'logo_force_auto_width_note',
			[
				'raw' => __('This will automatically set the logo image widget width to Inline(auto) ', 'she-header'),
				'type' => Controls_Manager::RAW_HTML,
				'content_classes' => 'elementor-panel-alert',
				'condition' => [
					'shrink_header_logo' => 'yes',
					'transparent!' => '',
				],
			]
		);

		$element->add_responsive_control(
			'custom_height_header_logo',
			[
				'label' => __('Logo Height (%)', 'she-header'),
				'type' => Controls_Manager::SLIDER,
				'default' => [
					'size' => 70,
					'unit' => '%',
				],
				'range' => [
					'%' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'size_units' => ['%'],
				'condition' => [
					'shrink_header_logo' => 'yes',
					'transparent!' => '',
				],
				'frontend_available' => true,
				'selectors' => [
					'.she-header .elementor-widget-theme-site-logo img' => 'max-width:{{SIZE}}{{UNIT}} !important;',
					'.she-header .logo img' => 'max-width:{{SIZE}}{{UNIT}} !important;',
					'.she-header-yes .elementor-widget-theme-site-logo' => 'width:auto !important;',
					'.she-header-yes .logo' => 'width:auto !important;',
				],
			]
		);

		$element->add_control(
			'logo_shrink_note',
			[
				'raw' => __('This feature is only compatible with the elementor site logo widget and image widgets with the class "logo"', 'she-header'),
				'type' => Controls_Manager::RAW_HTML,
				'content_classes' => 'elementor-panel-alert',
				'condition' => [
					'shrink_header_logo' => 'yes',
					'transparent!' => '',
				],
			]
		);

		// ---------------------------------- LOGO COLOR TOGGLE
		$element->add_control(
			'change_logo_color',
			[
				'label' => __('Change Logo Color', 'she-header'),
				'type' => Controls_Manager::SWITCHER,
				'separator' => 'before',
				'label_on' => __('On', 'she-header'),
				'label_off' => __('Off', 'she-header'),
				'return_value' => 'yes',
				'default' => '',
				'frontend_available' => true,
				'description' => __('Change the logo image color before or after the user reaches the scroll distance set above. Note: These might override other CSS Filter styles', 'she-header'),
				'condition' => [
					'transparent!' => '',
				],
			]
		);

		// ---------------------------------- LOGO COLOR BEFORE
		$element->add_control(
			'logo_color_before',
			[
				'label' => __('Before Scrolling', 'she-header'),
				'type' => Controls_Manager::SELECT,
				'options' => [
					'brightness(1) invert(0) !important' => __('None', 'she-header'),
					'brightness(0) invert(1) !important' => __('White', 'she-header'),
					'brightness(0) invert(0) !important' => __('Black', 'she-header'),
				],
				'default' => '',
				'frontend_available' => true,
				'condition' => [

					'change_logo_color' => 'yes',
					'transparent!' => '',
				],
				'selectors' => [
					'.she-header-yes:not(.she-header) .elementor-widget-theme-site-logo .elementor-widget-container' => '-webkit-filter:{{VALUE}}; filter:{{VALUE}};',
					'.she-header-yes:not(.she-header) .logo .elementor-widget-container' => '-webkit-filter:{{VALUE}}; filter:{{VALUE}};',
				],
			]
		);

		// ---------------------------------- LOGO COLOR AFTER

		$element->add_control(
			'logo_color_after',
			[
				'label' => __('After Scrolling', 'she-header'),
				'type' => Controls_Manager::SELECT,
				'options' => [
					'brightness(1) invert(0) !important' => __('None', 'she-header'),
					'brightness(0) invert(1) !important' => __('White', 'she-header'),
					'brightness(0) invert(0) !important' => __('Black', 'she-header'),
				],
				'default' => '',
				'frontend_available' => true,
				'condition' => [

					'change_logo_color' => 'yes',
					'transparent!' => '',
				],
				'selectors' => [
					'.she-header .elementor-widget-theme-site-logo .elementor-widget-container' => '-webkit-filter:{{VALUE}}; filter:{{VALUE}};',
					'.she-header .logo .elementor-widget-container' => '-webkit-filter:{{VALUE}}; filter:{{VALUE}};',
				],
			]
		);

		// ---------------------------------- LOGO COLOR NOTICE

		$element->add_control(
			'logo_color_note',
			[
				'raw' => __('This feature is only compatible with the elementor site logo widget and image widgets with the class "logo"', 'she-header'),
				'type' => Controls_Manager::RAW_HTML,
				'content_classes' => 'elementor-panel-alert',
				'condition' => [

					'change_logo_color' => 'yes',
					'transparent!' => '',
				],
			]
		);

		// ---------------------------------- BLUR BACKGROUND
		$element->add_control(
			'blur_bg',
			[
				'label' => __('Blur Background', 'she-header'),
				'type' => Controls_Manager::SWITCHER,
				'separator' => 'before',
				'label_on' => __('On', 'she-header'),
				'label_off' => __('Off', 'she-header'),
				'return_value' => 'yes',
				'default' => '',
				'frontend_available' => true,
				'condition' => [

					'transparent!' => '',
				],
				'description' => __('Add a modern blur effect to a semi-transparent header background color after scrolling', 'she-header'),
				'selectors' => [
					'{{WRAPPER}}.she-header' => 'backdrop-filter: blur(20px) saturate(180%); -webkit-backdrop-filter: blur(20px) saturate(180%);',
				],
			]
		);

		// ---------------------------------- HIDE HEADER ON SCROLL
		$element->add_control(
			'hide_header',
			[
				'label' => __('Hide header on scroll down', 'she-header'),
				'type' => Controls_Manager::SWITCHER,
				'separator' => 'before',
				'label_on' => __('On', 'she-header'),
				'label_off' => __('Off', 'she-header'),
				'return_value' => 'yes',
				'default' => '',
				'frontend_available' => true,
				'description' => __('Hides the header if scrolling down, shows header if scrolling up', 'she-header'),
				'prefix_class'  => 'she-header-hide-on-scroll-',
				'condition' => [

					'transparent!' => '',
				],
				'selectors' => [
					'{{WRAPPER}}.she-header-yes.headerup' => '
					transform: translateY(-200%) !important;
					-webkit-transform: translateY(-200%) !important;'
				],
			]
		);

		// ---------------------------------- SCROLL DISTANCE TO HIDE HEADER
		$element->add_responsive_control(
			'scroll_distance_hide_header',
			[
				'label' => __('Scroll Distance (Px)', 'she-header'),
				'type' => Controls_Manager::SLIDER,
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 2000,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 150,
					'step' => 1,
				],
				'description' => __('Choose the scroll distance to start hiding the header', 'she-header'),
				'frontend_available' => true,
				'condition' => [

					'hide_header' => 'yes',
					'transparent!' => '',
				],
			]
		);

		$element->add_control(
			'hide_header_note',
			[
				'raw' => __('This feature might conflict with entrance animations', 'she-header'),
				'type' => Controls_Manager::RAW_HTML,
				'content_classes' => 'elementor-panel-alert',
				'condition' => [
					'hide_header' => 'yes',
					'transparent!' => '',
				],
			]
		);

		$element->end_controls_section();
	}

	private function add_actions()
	{
		if (!function_exists('is_plugin_active')) {

			include_once(ABSPATH . 'wp-admin/includes/plugin.php');
		}

		// add She on sections
		if (is_plugin_active('elementor/elementor.php')) {
			add_action('elementor/element/section/section_effects/after_section_end', [$this, 'register_controls']);
		}

		add_action('elementor/frontend/after_enqueue_styles', [$this, 'enqueue_styles']);
		if (Elementor\Plugin::instance()->editor->is_edit_mode()) {
		} else {
			add_action('wp_enqueue_scripts', [$this, 'enqueue_scripts']);
		}

		// add She on containers
		if (is_plugin_active('elementor/elementor.php')) {
			add_action('elementor/element/container/section_effects/after_section_end', [$this, 'register_controls']);
		}
	}

	public function enqueue_styles()
	{
		$suffix = defined('SCRIPT_DEBUG') && SCRIPT_DEBUG ? '' : '.min';

		wp_enqueue_style(
			'she-header-style',
			SHE_HEADER_ASSETS_URL  . 'css/she-header-style' . '.css',
			[],
			SHE_HEADER_VERSION
		);
	}

	public function enqueue_scripts()
	{
		$suffix = defined('SCRIPT_DEBUG') && SCRIPT_DEBUG ? '' : '.min';

		wp_enqueue_script(
			'she-header',
			SHE_HEADER_URL . 'assets/js/she-header.js',
			[
				'jquery',
			],
			SHE_HEADER_VERSION,
			false
		);
	}
}